@@ -65,12 +65,6 @@ enum
   LAST_SIGNAL
 };
 
-static const GstElementDetails gst_cmml_enc_details =
-GST_ELEMENT_DETAILS ("CMML streams encoder",
-    "Codec/Encoder",
-    "Encodes CMML streams",
-    "Alessandro Decina <alessandro@nnva.org>");
-
 static GstStaticPadTemplate gst_cmml_enc_src_factory =
 GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
@@ -118,7 +112,9 @@ gst_cmml_enc_base_init (gpointer g_class)
       gst_static_pad_template_get (&gst_cmml_enc_sink_factory));
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_cmml_enc_src_factory));
-  gst_element_class_set_details (element_class, &gst_cmml_enc_details);
+  gst_element_class_set_details_simple (element_class, "CMML streams encoder",
+      "Codec/Encoder",
+      "Encodes CMML streams", "Alessandro Decina <alessandro@nnva.org>");
 }
 
 static void
@@ -589,9 +585,6 @@ gst_cmml_enc_push (GstCmmlEnc * enc, GstBuffer * buffer)
 {
   GstFlowReturn res;
 
-  /* FIXME: hack to make oggmux flush every cmml tag in its own page */
-  GST_BUFFER_DURATION (buffer) = G_MAXINT64;
-
   res = gst_pad_push (enc->srcpad, buffer);
   if (GST_FLOW_IS_FATAL (res))
     GST_WARNING_OBJECT (enc, "push returned: %s", gst_flow_get_name (res));
@@ -612,7 +605,7 @@ gst_cmml_enc_chain (GstPad * pad, GstBuffer * buffer)
 
   if (!gst_cmml_parser_parse_chunk (enc->parser,
           (gchar *) GST_BUFFER_DATA (buffer), GST_BUFFER_SIZE (buffer), &err)) {
-    GST_ELEMENT_ERROR (enc, STREAM, ENCODE, (NULL), (err->message));
+    GST_ELEMENT_ERROR (enc, STREAM, ENCODE, (NULL), ("%s", err->message));
     g_error_free (err);
     enc->flow_return = GST_FLOW_ERROR;
   }
