@@ -38,21 +38,19 @@
 #include <string.h>
 
 #include "gstjpegdec.h"
+#include "gstjpeg.h"
 #include <gst/video/video.h>
 #include "gst/gst-i18n-plugin.h"
 #include <jerror.h>
 
-static const GstElementDetails gst_jpeg_dec_details =
-GST_ELEMENT_DETAILS ("JPEG image decoder",
-    "Codec/Decoder/Image",
-    "Decode images from JPEG format",
-    "Wim Taymans <wim@fluendo.com>");
-
-#define MIN_WIDTH  16
+#define MIN_WIDTH  1
 #define MAX_WIDTH  65535
-#define MIN_HEIGHT 8
+#define MIN_HEIGHT 1
 #define MAX_HEIGHT 65535
 
+#define CINFO_GET_JPEGDEC(cinfo_ptr) \
+        (((struct GstJpegDecSourceMgr*)((cinfo_ptr)->src))->dec)
+
 #define JPEG_DEFAULT_IDCT_METHOD	JDCT_FASTEST
 
 enum
@@ -61,15 +59,18 @@ enum
   PROP_IDCT_METHOD
 };
 
-extern GType gst_idct_method_get_type (void);
-#define GST_TYPE_IDCT_METHOD (gst_idct_method_get_type())
-
+/* *INDENT-OFF* */
 static GstStaticPadTemplate gst_jpeg_dec_src_pad_template =
 GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS (GST_VIDEO_CAPS_YUV ("I420"))
+    GST_STATIC_CAPS (GST_VIDEO_CAPS_YUV ("I420") "; "
+        GST_VIDEO_CAPS_RGB "; " GST_VIDEO_CAPS_BGR "; "
+        GST_VIDEO_CAPS_RGBx "; " GST_VIDEO_CAPS_xRGB "; "
+        GST_VIDEO_CAPS_BGRx "; " GST_VIDEO_CAPS_xBGR "; "
+        GST_VIDEO_CAPS_GRAY8)
     );
+/* *INDENT-ON* */
 
 static GstStaticPadTemplate gst_jpeg_dec_sink_pad_template =
 GST_STATIC_PAD_TEMPLATE ("sink",
@@ -151,8 +152,7 @@ gst_jpeg_dec_finalize (GObject * object)
 
   jpeg_destroy_decompress (&dec->cinfo);
 
-  if (dec->tempbuf)
-    gst_buffer_unref (dec->tempbuf);
+  g_object_unref (dec->adapter);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -166,7 +166,9 @@ gst_jpeg_dec_base_init (gpointer g_class)
       gst_static_pad_template_get (&gst_jpeg_dec_src_pad_template));
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_jpeg_dec_sink_pad_template));
-  gst_element_class_set_details (element_class, &gst_jpeg_dec_details);
+  gst_element_class_set_details_simple (element_class, "JPEG image decoder",
+      "Codec/Decoder/Image",
+      "Decode images from JPEG format", "Wim Taymans <wim@fluendo.com>");
 }
 
 static void
@@ -199,20 +201,31 @@ gst_jpeg_dec_class_init (GstJpegDecClass * klass)
 static boolean
 gst_jpeg_dec_fill_input_buffer (j_decompress_ptr cinfo)
 {
-/*
-  struct GstJpegDecSourceMgr *src_mgr;
   GstJpegDec *dec;
+  guint av;
 
-  src_mgr = (struct GstJpegDecSourceMgr*) &cinfo->src;
-  dec = GST_JPEG_DEC (src_mgr->dec);
-*/
-  GST_DEBUG_OBJECT (CINFO_GET_JPEGDEC (cinfo), "fill_input_buffer");
-/*
-  g_return_val_if_fail (dec != NULL, TRUE);
+  dec = CINFO_GET_JPEGDEC (cinfo);
+  g_return_val_if_fail (dec != NULL, FALSE);
+
+  av = gst_adapter_available_fast (dec->adapter);
+  GST_DEBUG_OBJECT (dec, "fill_input_buffer: fast av=%u, remaining=%u", av,
+      dec->rem_img_len);
+
+  if (av == 0) {
+    GST_DEBUG_OBJECT (dec, "Out of data");
+    return FALSE;
+  }
+
+  if (dec->rem_img_len < av)
+    av = dec->rem_img_len;
+  dec->rem_img_len -= av;
+
+  g_free (dec->cur_buf);
+  dec->cur_buf = gst_adapter_take (dec->adapter, av);
+
+  cinfo->src->next_input_byte = dec->cur_buf;
+  cinfo->src->bytes_in_buffer = av;
 
-  src_mgr->pub.next_input_byte = GST_BUFFER_DATA (dec->tempbuf);
-  src_mgr->pub.bytes_in_buffer = GST_BUFFER_SIZE (dec->tempbuf);
-*/
   return TRUE;
 }
 
@@ -226,11 +239,29 @@ gst_jpeg_dec_init_source (j_decompress_ptr cinfo)
 static void
 gst_jpeg_dec_skip_input_data (j_decompress_ptr cinfo, glong num_bytes)
 {
-  GST_DEBUG_OBJECT (CINFO_GET_JPEGDEC (cinfo), "skip %ld bytes", num_bytes);
+  GstJpegDec *dec = CINFO_GET_JPEGDEC (cinfo);
+
+  GST_DEBUG_OBJECT (dec, "skip %ld bytes", num_bytes);
 
   if (num_bytes > 0 && cinfo->src->bytes_in_buffer >= num_bytes) {
     cinfo->src->next_input_byte += (size_t) num_bytes;
     cinfo->src->bytes_in_buffer -= (size_t) num_bytes;
+  } else if (num_bytes > 0) {
+    gint available;
+
+    num_bytes -= cinfo->src->bytes_in_buffer;
+    cinfo->src->next_input_byte += (size_t) cinfo->src->bytes_in_buffer;
+    cinfo->src->bytes_in_buffer = 0;
+
+    available = gst_adapter_available (dec->adapter);
+    if (available < num_bytes || available < dec->rem_img_len) {
+      GST_WARNING_OBJECT (dec, "Less bytes to skip than available in the "
+          "adapter or the remaining image length %ld < %d or %u",
+          num_bytes, available, dec->rem_img_len);
+    }
+    num_bytes = MIN (MIN (num_bytes, available), dec->rem_img_len);
+    gst_adapter_flush (dec->adapter, num_bytes);
+    dec->rem_img_len -= num_bytes;
   }
 }
 
@@ -314,89 +345,34 @@ gst_jpeg_dec_init (GstJpegDec * dec)
 
   /* init properties */
   dec->idct_method = JPEG_DEFAULT_IDCT_METHOD;
-}
 
-static inline gboolean
-is_jpeg_start_marker (const guint8 * data)
-{
-  return (data[0] == 0xff && data[1] == 0xd8);
-}
-
-static gboolean
-gst_jpeg_dec_find_jpeg_header (GstJpegDec * dec)
-{
-  const guint8 *data;
-  guint size;
-
-  data = GST_BUFFER_DATA (dec->tempbuf);
-  size = GST_BUFFER_SIZE (dec->tempbuf);
-
-  g_return_val_if_fail (size >= 2, FALSE);
-
-  while (!is_jpeg_start_marker (data) || data[2] != 0xff) {
-    const guint8 *marker;
-    GstBuffer *tmp;
-    guint off;
-
-    marker = memchr (data + 1, 0xff, size - 1 - 2);
-    if (marker == NULL) {
-      off = size - 1;           /* keep last byte */
-    } else {
-      off = marker - data;
-    }
-
-    tmp = gst_buffer_create_sub (dec->tempbuf, off, size - off);
-    gst_buffer_unref (dec->tempbuf);
-    dec->tempbuf = tmp;
-
-    data = GST_BUFFER_DATA (dec->tempbuf);
-    size = GST_BUFFER_SIZE (dec->tempbuf);
-
-    if (size < 2)
-      return FALSE;             /* wait for more data */
-  }
-
-  return TRUE;                  /* got header */
+  dec->adapter = gst_adapter_new ();
 }
 
 static gboolean
 gst_jpeg_dec_ensure_header (GstJpegDec * dec)
 {
-  g_return_val_if_fail (dec->tempbuf != NULL, FALSE);
+  gint av;
+  gint offset;
 
-check_header:
-
-  /* we need at least a start marker (0xff 0xd8)
-   *   and an end marker (0xff 0xd9) */
-  if (GST_BUFFER_SIZE (dec->tempbuf) <= 4) {
-    GST_DEBUG ("Not enough data");
-    return FALSE;               /* we need more data */
+  av = gst_adapter_available (dec->adapter);
+  /* we expect at least 4 bytes, first of which start marker */
+  offset = gst_adapter_masked_scan_uint32 (dec->adapter, 0xffffff00, 0xffd8ff00,
+      0, av);
+  if (G_UNLIKELY (offset < 0)) {
+    GST_DEBUG_OBJECT (dec, "No JPEG header in current buffer");
+    /* not found */
+    if (av > 4)
+      gst_adapter_flush (dec->adapter, av - 4);
+    return FALSE;
   }
 
-  if (!is_jpeg_start_marker (GST_BUFFER_DATA (dec->tempbuf))) {
-    GST_DEBUG ("Not a JPEG header, resyncing to header...");
-    if (!gst_jpeg_dec_find_jpeg_header (dec)) {
-      GST_DEBUG ("No JPEG header in current buffer");
-      return FALSE;             /* we need more data */
-    }
-    GST_DEBUG ("Found JPEG header");
-    goto check_header;          /* buffer might have changed */
-  }
+  GST_DEBUG_OBJECT (dec, "Found JPEG header");
+  gst_adapter_flush (dec->adapter, offset);
 
   return TRUE;
 }
 
-#if 0
-static gboolean
-gst_jpeg_dec_have_end_marker (GstJpegDec * dec)
-{
-  guint8 *data = GST_BUFFER_DATA (dec->tempbuf);
-  guint size = GST_BUFFER_SIZE (dec->tempbuf);
-
-  return (size > 2 && data && is_jpeg_end_marker (data + size - 2));
-}
-#endif
-
 static inline gboolean
 gst_jpeg_dec_parse_tag_has_entropy_segment (guint8 tag)
 {
@@ -405,93 +381,109 @@ gst_jpeg_dec_parse_tag_has_entropy_segment (guint8 tag)
   return FALSE;
 }
 
-/* returns image length in bytes if parsed 
+/* returns image length in bytes if parsed
  * successfully, otherwise 0 */
 static guint
 gst_jpeg_dec_parse_image_data (GstJpegDec * dec)
 {
-  guint8 *start, *data, *end;
   guint size;
+  gboolean resync;
+  GstAdapter *adapter = dec->adapter;
+  gint offset, noffset;
 
-  size = GST_BUFFER_SIZE (dec->tempbuf);
-  start = GST_BUFFER_DATA (dec->tempbuf);
-  end = start + size;
-  data = start;
+  size = gst_adapter_available (adapter);
 
-  g_return_val_if_fail (is_jpeg_start_marker (data), 0);
+  /* we expect at least 4 bytes, first of which start marker */
+  if (gst_adapter_masked_scan_uint32 (adapter, 0xffff0000, 0xffd80000, 0, 4))
+    return 0;
 
   GST_DEBUG ("Parsing jpeg image data (%u bytes)", size);
 
-  /* skip start marker */
-  data += 2;
+  GST_DEBUG ("Parse state: offset=%d, resync=%d, entropy len=%d",
+      dec->parse_offset, dec->parse_resync, dec->parse_entropy_len);
+
+  /* offset is 2 less than actual offset;
+   * - adapter needs at least 4 bytes for scanning,
+   * - start and end marker ensure at least that much
+   */
+  /* resume from state offset */
+  offset = dec->parse_offset;
 
   while (1) {
     guint frame_len;
-    gboolean resync;
+    guint32 value;
 
-    /* do we need to resync? */
-    resync = (*data != 0xff);
-    if (resync) {
-      GST_DEBUG ("Lost sync at 0x%08" G_GINT64_MODIFIER "x, resyncing",
-          (gint64) (data - start));
-      /* at the very least we expect 0xff 0xNN, thus end-1 */
-      while (*data != 0xff && data < end - 1)
-        ++data;
-      if (G_UNLIKELY (*data != 0xff)) {
-        GST_DEBUG ("at end of input and no next marker found, need more data");
-        return 0;
-      }
+    noffset =
+        gst_adapter_masked_scan_uint32_peek (adapter, 0x0000ff00, 0x0000ff00,
+        offset, size - offset, &value);
+    /* lost sync if 0xff marker not where expected */
+    if ((resync = (noffset != offset))) {
+      GST_DEBUG ("Lost sync at 0x%08x, resyncing", offset + 2);
     }
+    /* may have marker, but could have been resyncng */
+    resync = resync || dec->parse_resync;
     /* Skip over extra 0xff */
-    while (*data == 0xff && data < end)
-      ++data;
+    while ((noffset >= 0) && ((value & 0xff) == 0xff)) {
+      noffset++;
+      noffset =
+          gst_adapter_masked_scan_uint32_peek (adapter, 0x0000ff00, 0x0000ff00,
+          noffset, size - noffset, &value);
+    }
     /* enough bytes left for marker? (we need 0xNN after the 0xff) */
-    if (data >= end) {
+    if (noffset < 0) {
       GST_DEBUG ("at end of input and no EOI marker found, need more data");
-      return 0;
+      goto need_more_data;
     }
 
-    if (*data == 0xd9) {
-      GST_DEBUG ("0x%08" G_GINT64_MODIFIER "x: EOI marker",
-          (gint64) (data - start));
-      return (data - start + 1);
+    /* now lock on the marker we found */
+    offset = noffset;
+    value = value & 0xff;
+    if (value == 0xd9) {
+      GST_DEBUG ("0x%08x: EOI marker", offset + 2);
+      /* clear parse state */
+      dec->parse_resync = FALSE;
+      dec->parse_offset = 0;
+      return (offset + 4);
     }
 
-    if (*data >= 0xd0 && *data <= 0xd7)
+    if (value >= 0xd0 && value <= 0xd7)
       frame_len = 0;
-    else if (data >= end - 2)
-      return 0;
-    else
-      frame_len = GST_READ_UINT16_BE (data + 1);
-    GST_DEBUG ("0x%08" G_GINT64_MODIFIER "x: tag %02x, frame_len=%u",
-        (gint64) (data - start - 1), *data, frame_len);
+    else {
+      /* peek tag and subsequent length */
+      if (offset + 2 + 4 > size)
+        goto need_more_data;
+      else
+        gst_adapter_masked_scan_uint32_peek (adapter, 0x0, 0x0, offset + 2, 4,
+            &frame_len);
+      frame_len = frame_len & 0xffff;
+    }
+    GST_DEBUG ("0x%08x: tag %02x, frame_len=%u", offset + 2, value, frame_len);
     /* the frame length includes the 2 bytes for the length; here we want at
-     * least 2 more bytes at the end for an end marker, thus end-2 */
-    if (data + 1 + frame_len >= end - 2) {
-      if (resync) {
-        GST_DEBUG ("not a valid sync (not enough data).");
-        /* Since *data != 0xff, the next iteration will go into resync again. */
-        continue;
-      }
-      /* theoretically we could have lost sync and not really need more
-       * data, but that's just tough luck and a broken image then */
-      GST_DEBUG ("at end of input and no EOI marker found, need more data");
-      return 0;
+     * least 2 more bytes at the end for an end marker */
+    if (offset + 2 + 2 + frame_len + 2 > size) {
+      goto need_more_data;
     }
 
-    if (gst_jpeg_dec_parse_tag_has_entropy_segment (*data)) {
-      guint8 *d2 = data + 1 + frame_len;
-      guint eseglen = 0;
+    if (gst_jpeg_dec_parse_tag_has_entropy_segment (value)) {
+      guint eseglen = dec->parse_entropy_len;
 
-      GST_DEBUG ("0x%08" G_GINT64_MODIFIER "x: finding entropy segment length",
-          (gint64) (data - start - 1));
+      GST_DEBUG ("0x%08x: finding entropy segment length", offset + 2);
+      noffset = offset + 2 + frame_len + dec->parse_entropy_len;
       while (1) {
-        if (d2[eseglen] == 0xff && d2[eseglen + 1] != 0x00)
+        noffset = gst_adapter_masked_scan_uint32_peek (adapter, 0x0000ff00,
+            0x0000ff00, noffset, size - noffset, &value);
+        if (noffset < 0) {
+          /* need more data */
+          dec->parse_entropy_len = size - offset - 4 - frame_len - 2;
+          goto need_more_data;
+        }
+        if ((value & 0xff) != 0x00) {
+          eseglen = noffset - offset - frame_len - 2;
           break;
-        if (d2 + eseglen >= end - 1)
-          return 0;             /* need more data */
-        ++eseglen;
+        }
+        noffset++;
       }
+      dec->parse_entropy_len = 0;
       frame_len += eseglen;
       GST_DEBUG ("entropy segment length=%u => frame_len=%u", eseglen,
           frame_len);
@@ -499,15 +491,26 @@ gst_jpeg_dec_parse_image_data (GstJpegDec * dec)
     if (resync) {
       /* check if we will still be in sync if we interpret
        * this as a sync point and skip this frame */
-      if (data[2 + frame_len] != 0xff) {
+      noffset = offset + frame_len + 2;
+      noffset = gst_adapter_masked_scan_uint32 (adapter, 0x0000ff00, 0x0000ff00,
+          noffset, 4);
+      if (noffset < 0) {
         /* ignore and continue resyncing until we hit the end
          * of our data or find a sync point that looks okay */
         continue;
       }
-      GST_DEBUG ("found sync at %p", data - size);
+      GST_DEBUG ("found sync at 0x%x", offset + 2);
     }
 
-    data += 1 + frame_len;
+    offset += frame_len + 2;
+  }
+
+  /* EXITS */
+need_more_data:
+  {
+    dec->parse_offset = offset;
+    dec->parse_resync = resync;
+    return 0;
   }
 }
 
@@ -717,9 +720,87 @@ gst_jpeg_dec_ensure_buffers (GstJpegDec * dec, guint maxrowbytes)
   return TRUE;
 }
 
+static void
+gst_jpeg_dec_decode_grayscale (GstJpegDec * dec, guchar * base[1],
+    guint width, guint height, guint pstride, guint rstride)
+{
+  guchar *rows[16];
+  guchar **scanarray[1] = { rows };
+  gint i, j, k;
+  gint lines;
+
+  GST_DEBUG_OBJECT (dec, "indirect decoding of grayscale");
+
+  if (G_UNLIKELY (!gst_jpeg_dec_ensure_buffers (dec, GST_ROUND_UP_32 (width))))
+    return;
+
+  memcpy (rows, dec->idr_y, 16 * sizeof (gpointer));
+
+  i = 0;
+  while (i < height) {
+    lines = jpeg_read_raw_data (&dec->cinfo, scanarray, DCTSIZE);
+    if (G_LIKELY (lines > 0)) {
+      for (j = 0; (j < DCTSIZE) && (i < height); j++, i++) {
+        gint p;
+
+        p = 0;
+        for (k = 0; k < width; k++) {
+          base[0][p] = rows[j][k];
+          p += pstride;
+        }
+        base[0] += rstride;
+      }
+    } else {
+      GST_INFO_OBJECT (dec, "jpeg_read_raw_data() returned 0");
+    }
+  }
+}
+
+static void
+gst_jpeg_dec_decode_rgb (GstJpegDec * dec, guchar * base[3],
+    guint width, guint height, guint pstride, guint rstride)
+{
+  guchar *r_rows[16], *g_rows[16], *b_rows[16];
+  guchar **scanarray[3] = { r_rows, g_rows, b_rows };
+  gint i, j, k;
+  gint lines;
+
+  GST_DEBUG_OBJECT (dec, "indirect decoding of RGB");
+
+  if (G_UNLIKELY (!gst_jpeg_dec_ensure_buffers (dec, GST_ROUND_UP_32 (width))))
+    return;
+
+  memcpy (r_rows, dec->idr_y, 16 * sizeof (gpointer));
+  memcpy (g_rows, dec->idr_u, 16 * sizeof (gpointer));
+  memcpy (b_rows, dec->idr_v, 16 * sizeof (gpointer));
+
+  i = 0;
+  while (i < height) {
+    lines = jpeg_read_raw_data (&dec->cinfo, scanarray, DCTSIZE);
+    if (G_LIKELY (lines > 0)) {
+      for (j = 0; (j < DCTSIZE) && (i < height); j++, i++) {
+        gint p;
+
+        p = 0;
+        for (k = 0; k < width; k++) {
+          base[0][p] = r_rows[j][k];
+          base[1][p] = g_rows[j][k];
+          base[2][p] = b_rows[j][k];
+          p += pstride;
+        }
+        base[0] += rstride;
+        base[1] += rstride;
+        base[2] += rstride;
+      }
+    } else {
+      GST_INFO_OBJECT (dec, "jpeg_read_raw_data() returned 0");
+    }
+  }
+}
+
 static void
 gst_jpeg_dec_decode_indirect (GstJpegDec * dec, guchar * base[3],
-    guchar * last[3], guint width, guint height, gint r_v, gint r_h)
+    guchar * last[3], guint width, guint height, gint r_v, gint r_h, gint comp)
 {
   guchar *y_rows[16], *u_rows[16], *v_rows[16];
   guchar **scanarray[3] = { y_rows, u_rows, v_rows };
@@ -736,33 +817,44 @@ gst_jpeg_dec_decode_indirect (GstJpegDec * dec, guchar * base[3],
   memcpy (u_rows, dec->idr_u, 16 * sizeof (gpointer));
   memcpy (v_rows, dec->idr_v, 16 * sizeof (gpointer));
 
+  /* fill chroma components for grayscale */
+  if (comp == 1) {
+    GST_DEBUG_OBJECT (dec, "grayscale, filling chroma");
+    for (i = 0; i < 16; i++) {
+      memset (u_rows[i], GST_ROUND_UP_32 (width), 0x80);
+      memset (v_rows[i], GST_ROUND_UP_32 (width), 0x80);
+    }
+  }
+
   for (i = 0; i < height; i += r_v * DCTSIZE) {
     lines = jpeg_read_raw_data (&dec->cinfo, scanarray, r_v * DCTSIZE);
     if (G_LIKELY (lines > 0)) {
       for (j = 0, k = 0; j < (r_v * DCTSIZE); j += r_v, k++) {
-        memcpy (base[0], y_rows[j], I420_Y_ROWSTRIDE (width));
-        if (G_LIKELY (base[0] < last[0]))
+        if (G_LIKELY (base[0] <= last[0])) {
+          memcpy (base[0], y_rows[j], I420_Y_ROWSTRIDE (width));
           base[0] += I420_Y_ROWSTRIDE (width);
-        if (r_v == 2) {
-          memcpy (base[0], y_rows[j + 1], I420_Y_ROWSTRIDE (width));
-          if (G_LIKELY (base[0] < last[0]))
-            base[0] += I420_Y_ROWSTRIDE (width);
         }
-        if (r_h == 2) {
-          memcpy (base[1], u_rows[k], I420_U_ROWSTRIDE (width));
-          memcpy (base[2], v_rows[k], I420_V_ROWSTRIDE (width));
-        } else if (r_h == 1) {
-          hresamplecpy1 (base[1], u_rows[k], I420_U_ROWSTRIDE (width));
-          hresamplecpy1 (base[2], v_rows[k], I420_V_ROWSTRIDE (width));
-        } else {
-          /* FIXME: implement (at least we avoid crashing by doing nothing) */
+        if (r_v == 2) {
+          if (G_LIKELY (base[0] <= last[0])) {
+            memcpy (base[0], y_rows[j + 1], I420_Y_ROWSTRIDE (width));
+            base[0] += I420_Y_ROWSTRIDE (width);
+          }
+        }
+        if (G_LIKELY (base[1] <= last[1] && base[2] <= last[2])) {
+          if (r_h == 2) {
+            memcpy (base[1], u_rows[k], I420_U_ROWSTRIDE (width));
+            memcpy (base[2], v_rows[k], I420_V_ROWSTRIDE (width));
+          } else if (r_h == 1) {
+            hresamplecpy1 (base[1], u_rows[k], I420_U_ROWSTRIDE (width));
+            hresamplecpy1 (base[2], v_rows[k], I420_V_ROWSTRIDE (width));
+          } else {
+            /* FIXME: implement (at least we avoid crashing by doing nothing) */
+          }
         }
 
         if (r_v == 2 || (k & 1) != 0) {
-          if (G_LIKELY (base[1] < last[1] && base[2] < last[2])) {
-            base[1] += I420_U_ROWSTRIDE (width);
-            base[2] += I420_V_ROWSTRIDE (width);
-          }
+          base[1] += I420_U_ROWSTRIDE (width);
+          base[2] += I420_V_ROWSTRIDE (width);
         }
       }
     } else {
@@ -930,13 +1022,130 @@ gst_jpeg_dec_do_qos (GstJpegDec * dec, GstClockTime timestamp)
   return TRUE;
 }
 
+static void
+gst_jpeg_dec_negotiate (GstJpegDec * dec, gint width, gint height, gint clrspc)
+{
+  GstCaps *caps;
+  GstVideoFormat format;
+
+  if (G_UNLIKELY (width == dec->caps_width && height == dec->caps_height &&
+          dec->framerate_numerator == dec->caps_framerate_numerator &&
+          dec->framerate_denominator == dec->caps_framerate_denominator &&
+          clrspc == dec->clrspc))
+    return;
+
+  /* framerate == 0/1 is a still frame */
+  if (dec->framerate_denominator == 0) {
+    dec->framerate_numerator = 0;
+    dec->framerate_denominator = 1;
+  }
+
+  /* calculate or assume an average frame duration for QoS purposes */
+  GST_OBJECT_LOCK (dec);
+  if (dec->framerate_numerator != 0) {
+    dec->qos_duration = gst_util_uint64_scale (GST_SECOND,
+        dec->framerate_denominator, dec->framerate_numerator);
+  } else {
+    /* if not set just use 25fps */
+    dec->qos_duration = gst_util_uint64_scale (GST_SECOND, 1, 25);
+  }
+  GST_OBJECT_UNLOCK (dec);
+
+  if (dec->cinfo.jpeg_color_space == JCS_RGB) {
+    gint i;
+    GstCaps *allowed_caps;
+
+    GST_DEBUG_OBJECT (dec, "selecting RGB format");
+    /* retrieve allowed caps, and find the first one that reasonably maps
+     * to the parameters of the colourspace */
+    caps = gst_pad_get_allowed_caps (dec->srcpad);
+    if (!caps) {
+      GST_DEBUG_OBJECT (dec, "... but no peer, using template caps");
+      /* need to copy because get_allowed_caps returns a ref,
+       * and get_pad_template_caps doesn't */
+      caps = gst_caps_copy (gst_pad_get_pad_template_caps (dec->srcpad));
+    }
+    /* avoid lists of fourcc, etc */
+    allowed_caps = gst_caps_normalize (caps);
+    gst_caps_unref (caps);
+    caps = NULL;
+    GST_LOG_OBJECT (dec, "allowed source caps %" GST_PTR_FORMAT, allowed_caps);
+
+    for (i = 0; i < gst_caps_get_size (allowed_caps); i++) {
+      if (caps)
+        gst_caps_unref (caps);
+      caps = gst_caps_copy_nth (allowed_caps, i);
+      /* sigh, ds and _parse_caps need fixed caps for parsing, fixate */
+      gst_pad_fixate_caps (dec->srcpad, caps);
+      GST_LOG_OBJECT (dec, "checking caps %" GST_PTR_FORMAT, caps);
+      if (!gst_video_format_parse_caps (caps, &format, NULL, NULL))
+        continue;
+      /* we'll settle for the first (preferred) downstream rgb format */
+      if (gst_video_format_is_rgb (format))
+        break;
+      /* default fall-back */
+      format = GST_VIDEO_FORMAT_RGB;
+    }
+    if (caps)
+      gst_caps_unref (caps);
+    gst_caps_unref (allowed_caps);
+    caps = gst_video_format_new_caps (format, width, height,
+        dec->framerate_numerator, dec->framerate_denominator, 1, 1);
+    dec->outsize = gst_video_format_get_size (format, width, height);
+    /* some format info */
+    dec->offset[0] =
+        gst_video_format_get_component_offset (format, 0, width, height);
+    dec->offset[1] =
+        gst_video_format_get_component_offset (format, 1, width, height);
+    dec->offset[2] =
+        gst_video_format_get_component_offset (format, 2, width, height);
+    /* equal for all components */
+    dec->stride = gst_video_format_get_row_stride (format, 0, width);
+    dec->inc = gst_video_format_get_pixel_stride (format, 0);
+  } else if (dec->cinfo.jpeg_color_space == JCS_GRAYSCALE) {
+    /* TODO is anything else then 8bit supported in jpeg? */
+    format = GST_VIDEO_FORMAT_GRAY8;
+    caps = gst_video_format_new_caps (format, width, height,
+        dec->framerate_numerator, dec->framerate_denominator, 1, 1);
+    dec->outsize = gst_video_format_get_size (format, width, height);
+    dec->offset[0] =
+        gst_video_format_get_component_offset (format, 0, width, height);
+    dec->stride = gst_video_format_get_row_stride (format, 0, width);
+    dec->inc = gst_video_format_get_pixel_stride (format, 0);
+  } else {
+    /* go for plain and simple I420 */
+    /* TODO other YUV cases ? */
+    caps = gst_caps_new_simple ("video/x-raw-yuv",
+        "format", GST_TYPE_FOURCC, GST_MAKE_FOURCC ('I', '4', '2', '0'),
+        "width", G_TYPE_INT, width, "height", G_TYPE_INT, height,
+        "framerate", GST_TYPE_FRACTION, dec->framerate_numerator,
+        dec->framerate_denominator, NULL);
+    dec->outsize = I420_SIZE (width, height);
+  }
+
+  GST_DEBUG_OBJECT (dec, "setting caps %" GST_PTR_FORMAT, caps);
+  GST_DEBUG_OBJECT (dec, "max_v_samp_factor=%d", dec->cinfo.max_v_samp_factor);
+  GST_DEBUG_OBJECT (dec, "max_h_samp_factor=%d", dec->cinfo.max_h_samp_factor);
+
+  gst_pad_set_caps (dec->srcpad, caps);
+  gst_caps_unref (caps);
+
+  dec->caps_width = width;
+  dec->caps_height = height;
+  dec->caps_framerate_numerator = dec->framerate_numerator;
+  dec->caps_framerate_denominator = dec->framerate_denominator;
+}
+
 static GstFlowReturn
 gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buf)
 {
   GstFlowReturn ret = GST_FLOW_OK;
   GstJpegDec *dec;
   GstBuffer *outbuf = NULL;
-  guchar *data, *outdata;
+#ifndef GST_DISABLE_GST_DEBUG
+  guchar *data;
+#endif
+  guchar *outdata;
   guchar *base[3], *last[3];
   guint img_len, outsize;
   gint width, height;
@@ -955,17 +1164,13 @@ gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buf)
   if (GST_BUFFER_IS_DISCONT (buf)) {
     GST_DEBUG_OBJECT (dec, "buffer has DISCONT flag set");
     dec->discont = TRUE;
-    if (!dec->packetized && dec->tempbuf != NULL) {
+    if (!dec->packetized && gst_adapter_available (dec->adapter)) {
       GST_WARNING_OBJECT (dec, "DISCONT buffer in non-packetized mode, bad");
-      gst_buffer_replace (&dec->tempbuf, NULL);
+      gst_adapter_clear (dec->adapter);
     }
   }
 
-  if (dec->tempbuf) {
-    dec->tempbuf = gst_buffer_join (dec->tempbuf, buf);
-  } else {
-    dec->tempbuf = buf;
-  }
+  gst_adapter_push (dec->adapter, buf);
   buf = NULL;
 
   /* If we are non-packetized and know the total incoming size in bytes,
@@ -973,10 +1178,10 @@ gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buf)
 
   if (!dec->packetized && (dec->segment.format == GST_FORMAT_BYTES) &&
       (dec->segment.stop != -1) &&
-      (GST_BUFFER_SIZE (dec->tempbuf) < dec->segment.stop)) {
+      (gst_adapter_available (dec->adapter) < dec->segment.stop)) {
     /* We assume that non-packetized input in bytes is *one* single jpeg image */
     GST_DEBUG ("Non-packetized mode. Got %d bytes, need %" G_GINT64_FORMAT,
-        GST_BUFFER_SIZE (dec->tempbuf), dec->segment.stop);
+        gst_adapter_available (dec->adapter), dec->segment.stop);
     goto need_more_data;
   }
 
@@ -988,7 +1193,7 @@ gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buf)
    * do some sanity checking instead of parsing all of 
    * the jpeg data */
   if (dec->packetized) {
-    img_len = GST_BUFFER_SIZE (dec->tempbuf);
+    img_len = gst_adapter_available (dec->adapter);
   } else {
     /* Parse jpeg image to handle jpeg input that
      * is not aligned to buffer boundaries */
@@ -1002,11 +1207,16 @@ gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buf)
   if (dec->packetized && !gst_jpeg_dec_do_qos (dec, timestamp))
     goto skip_decoding;
 
-  data = (guchar *) GST_BUFFER_DATA (dec->tempbuf);
   GST_LOG_OBJECT (dec, "image size = %u", img_len);
 
-  dec->jsrc.pub.next_input_byte = data;
-  dec->jsrc.pub.bytes_in_buffer = img_len;
+#ifndef GST_DISABLE_GST_DEBUG
+  data = (guint8 *) gst_adapter_peek (dec->adapter, 4);
+  GST_LOG_OBJECT (dec, "reading header %02x %02x %02x %02x", data[0], data[1],
+      data[2], data[3]);
+#endif
+
+  dec->rem_img_len = img_len;
+  gst_jpeg_dec_fill_input_buffer (&dec->cinfo);
 
   if (setjmp (dec->jerr.setjmp_buffer)) {
     code = dec->jerr.pub.msg_code;
@@ -1018,9 +1228,6 @@ gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buf)
     goto decode_error;
   }
 
-  GST_LOG_OBJECT (dec, "reading header %02x %02x %02x %02x", data[0], data[1],
-      data[2], data[3]);
-
   /* read header */
   hdr_ok = jpeg_read_header (&dec->cinfo, TRUE);
   if (G_UNLIKELY (hdr_ok != JPEG_HEADER_OK)) {
@@ -1034,6 +1241,15 @@ gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buf)
   GST_LOG_OBJECT (dec, "num_components=%d", dec->cinfo.num_components);
   GST_LOG_OBJECT (dec, "jpeg_color_space=%d", dec->cinfo.jpeg_color_space);
 
+  if (dec->cinfo.num_components > 3)
+    goto components_not_supported;
+
+  /* verify color space expectation to avoid going *boom* or bogus output */
+  if (dec->cinfo.jpeg_color_space != JCS_YCbCr &&
+      dec->cinfo.jpeg_color_space != JCS_GRAYSCALE &&
+      dec->cinfo.jpeg_color_space != JCS_RGB)
+    goto unsupported_colorspace;
+
 #ifndef GST_DISABLE_GST_DEBUG
   {
     gint i;
@@ -1050,7 +1266,7 @@ gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buf)
   /* prepare for raw output */
   dec->cinfo.do_fancy_upsampling = FALSE;
   dec->cinfo.do_block_smoothing = FALSE;
-  dec->cinfo.out_color_space = JCS_YCbCr;
+  dec->cinfo.out_color_space = dec->cinfo.jpeg_color_space;
   dec->cinfo.dct_method = dec->idct_method;
   dec->cinfo.raw_data_out = TRUE;
 
@@ -1060,6 +1276,30 @@ gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buf)
     GST_WARNING_OBJECT (dec, "failed to start decompression cycle");
   }
 
+  /* sanity checks to get safe and reasonable output */
+  switch (dec->cinfo.jpeg_color_space) {
+    case JCS_GRAYSCALE:
+      if (dec->cinfo.num_components != 1)
+        goto invalid_yuvrgbgrayscale;
+      break;
+    case JCS_RGB:
+      if (dec->cinfo.num_components != 3 || dec->cinfo.max_v_samp_factor > 1 ||
+          dec->cinfo.max_h_samp_factor > 1)
+        goto invalid_yuvrgbgrayscale;
+      break;
+    case JCS_YCbCr:
+      if (dec->cinfo.num_components != 3 ||
+          r_v > 2 || r_v < dec->cinfo.comp_info[0].v_samp_factor ||
+          r_v < dec->cinfo.comp_info[1].v_samp_factor ||
+          r_h < dec->cinfo.comp_info[0].h_samp_factor ||
+          r_h < dec->cinfo.comp_info[1].h_samp_factor)
+        goto invalid_yuvrgbgrayscale;
+      break;
+    default:
+      g_assert_not_reached ();
+      break;
+  }
+
   width = dec->cinfo.output_width;
   height = dec->cinfo.output_height;
 
@@ -1067,50 +1307,7 @@ gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buf)
           height < MIN_HEIGHT || height > MAX_HEIGHT))
     goto wrong_size;
 
-  if (G_UNLIKELY (width != dec->caps_width || height != dec->caps_height ||
-          dec->framerate_numerator != dec->caps_framerate_numerator ||
-          dec->framerate_denominator != dec->caps_framerate_denominator)) {
-    GstCaps *caps;
-
-    /* framerate == 0/1 is a still frame */
-    if (dec->framerate_denominator == 0) {
-      dec->framerate_numerator = 0;
-      dec->framerate_denominator = 1;
-    }
-
-    /* calculate or assume an average frame duration for QoS purposes */
-    GST_OBJECT_LOCK (dec);
-    if (dec->framerate_numerator != 0) {
-      dec->qos_duration = gst_util_uint64_scale (GST_SECOND,
-          dec->framerate_denominator, dec->framerate_numerator);
-    } else {
-      /* if not set just use 25fps */
-      dec->qos_duration = gst_util_uint64_scale (GST_SECOND, 1, 25);
-    }
-    GST_OBJECT_UNLOCK (dec);
-
-    caps = gst_caps_new_simple ("video/x-raw-yuv",
-        "format", GST_TYPE_FOURCC, GST_MAKE_FOURCC ('I', '4', '2', '0'),
-        "width", G_TYPE_INT, width,
-        "height", G_TYPE_INT, height,
-        "framerate", GST_TYPE_FRACTION, dec->framerate_numerator,
-        dec->framerate_denominator, NULL);
-
-    GST_DEBUG_OBJECT (dec, "setting caps %" GST_PTR_FORMAT, caps);
-    GST_DEBUG_OBJECT (dec, "max_v_samp_factor=%d",
-        dec->cinfo.max_v_samp_factor);
-    GST_DEBUG_OBJECT (dec, "max_h_samp_factor=%d",
-        dec->cinfo.max_h_samp_factor);
-
-    gst_pad_set_caps (dec->srcpad, caps);
-    gst_caps_unref (caps);
-
-    dec->caps_width = width;
-    dec->caps_height = height;
-    dec->caps_framerate_numerator = dec->framerate_numerator;
-    dec->caps_framerate_denominator = dec->framerate_denominator;
-    dec->outsize = I420_SIZE (width, height);
-  }
+  gst_jpeg_dec_negotiate (dec, width, height, dec->cinfo.jpeg_color_space);
 
   ret = gst_pad_alloc_buffer_and_set_caps (dec->srcpad, GST_BUFFER_OFFSET_NONE,
       dec->outsize, GST_PAD_CAPS (dec->srcpad), &outbuf);
@@ -1143,43 +1340,55 @@ gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buf)
   }
   GST_BUFFER_DURATION (outbuf) = duration;
 
-  /* mind the swap, jpeglib outputs blue chroma first
-   * ensonic: I see no swap?
-   */
-  base[0] = outdata + I420_Y_OFFSET (width, height);
-  base[1] = outdata + I420_U_OFFSET (width, height);
-  base[2] = outdata + I420_V_OFFSET (width, height);
-
-  /* make sure we don't make jpeglib write beyond our buffer,
-   * which might happen if (height % (r_v*DCTSIZE)) != 0 */
-  last[0] = base[0] + (I420_Y_ROWSTRIDE (width) * (height - 1));
-  last[1] =
-      base[1] + (I420_U_ROWSTRIDE (width) * ((GST_ROUND_UP_2 (height) / 2) -
-          1));
-  last[2] =
-      base[2] + (I420_V_ROWSTRIDE (width) * ((GST_ROUND_UP_2 (height) / 2) -
-          1));
-
-  GST_LOG_OBJECT (dec, "decompressing (reqired scanline buffer height = %u)",
-      dec->cinfo.rec_outbuf_height);
-
-  /* For some widths jpeglib requires more horizontal padding than I420 
-   * provides. In those cases we need to decode into separate buffers and then
-   * copy over the data into our final picture buffer, otherwise jpeglib might
-   * write over the end of a line into the beginning of the next line,
-   * resulting in blocky artifacts on the left side of the picture. */
-  if (G_UNLIKELY (width % (dec->cinfo.max_h_samp_factor * DCTSIZE) != 0
-          || dec->cinfo.comp_info[0].h_samp_factor != 2
-          || dec->cinfo.comp_info[1].h_samp_factor != 1
-          || dec->cinfo.comp_info[2].h_samp_factor != 1)) {
-    GST_CAT_LOG_OBJECT (GST_CAT_PERFORMANCE, dec,
-        "indirect decoding using extra buffer copy");
-    gst_jpeg_dec_decode_indirect (dec, base, last, width, height, r_v, r_h);
+  if (dec->cinfo.jpeg_color_space == JCS_RGB) {
+    base[0] = outdata + dec->offset[0];
+    base[1] = outdata + dec->offset[1];
+    base[2] = outdata + dec->offset[2];
+    gst_jpeg_dec_decode_rgb (dec, base, width, height, dec->inc, dec->stride);
+  } else if (dec->cinfo.jpeg_color_space == JCS_GRAYSCALE) {
+    base[0] = outdata + dec->offset[0];
+    gst_jpeg_dec_decode_grayscale (dec, base, width, height, dec->inc,
+        dec->stride);
   } else {
-    ret = gst_jpeg_dec_decode_direct (dec, base, last, width, height);
+    /* mind the swap, jpeglib outputs blue chroma first
+     * ensonic: I see no swap?
+     */
+    base[0] = outdata + I420_Y_OFFSET (width, height);
+    base[1] = outdata + I420_U_OFFSET (width, height);
+    base[2] = outdata + I420_V_OFFSET (width, height);
 
-    if (G_UNLIKELY (ret != GST_FLOW_OK))
-      goto decode_direct_failed;
+    /* make sure we don't make jpeglib write beyond our buffer,
+     * which might happen if (height % (r_v*DCTSIZE)) != 0 */
+    last[0] = base[0] + (I420_Y_ROWSTRIDE (width) * (height - 1));
+    last[1] =
+        base[1] + (I420_U_ROWSTRIDE (width) * ((GST_ROUND_UP_2 (height) / 2) -
+            1));
+    last[2] =
+        base[2] + (I420_V_ROWSTRIDE (width) * ((GST_ROUND_UP_2 (height) / 2) -
+            1));
+
+    GST_LOG_OBJECT (dec, "decompressing (reqired scanline buffer height = %u)",
+        dec->cinfo.rec_outbuf_height);
+
+    /* For some widths jpeglib requires more horizontal padding than I420 
+     * provides. In those cases we need to decode into separate buffers and then
+     * copy over the data into our final picture buffer, otherwise jpeglib might
+     * write over the end of a line into the beginning of the next line,
+     * resulting in blocky artifacts on the left side of the picture. */
+    if (G_UNLIKELY (width % (dec->cinfo.max_h_samp_factor * DCTSIZE) != 0
+            || dec->cinfo.comp_info[0].h_samp_factor != 2
+            || dec->cinfo.comp_info[1].h_samp_factor != 1
+            || dec->cinfo.comp_info[2].h_samp_factor != 1)) {
+      GST_CAT_LOG_OBJECT (GST_CAT_PERFORMANCE, dec,
+          "indirect decoding using extra buffer copy");
+      gst_jpeg_dec_decode_indirect (dec, base, last, width, height, r_v, r_h,
+          dec->cinfo.num_components);
+    } else {
+      ret = gst_jpeg_dec_decode_direct (dec, base, last, width, height);
+
+      if (G_UNLIKELY (ret != GST_FLOW_OK))
+        goto decode_direct_failed;
+    }
   }
 
   GST_LOG_OBJECT (dec, "decompressing finished");
@@ -1219,16 +1428,7 @@ gst_jpeg_dec_chain (GstPad * pad, GstBuffer * buf)
 
 skip_decoding:
 done:
-  if (GST_BUFFER_SIZE (dec->tempbuf) == img_len) {
-    gst_buffer_unref (dec->tempbuf);
-    dec->tempbuf = NULL;
-  } else {
-    GstBuffer *buf = gst_buffer_create_sub (dec->tempbuf, img_len,
-        GST_BUFFER_SIZE (dec->tempbuf) - img_len);
-
-    gst_buffer_unref (dec->tempbuf);
-    dec->tempbuf = buf;
-  }
+  gst_adapter_flush (dec->adapter, dec->rem_img_len);
 
 exit:
 
@@ -1299,6 +1499,27 @@ drop_buffer:
     ret = GST_FLOW_OK;
     goto exit;
   }
+components_not_supported:
+  {
+    GST_ELEMENT_ERROR (dec, STREAM, DECODE, (NULL),
+        ("more components than supported: %d > 3", dec->cinfo.num_components));
+    ret = GST_FLOW_ERROR;
+    goto done;
+  }
+unsupported_colorspace:
+  {
+    GST_ELEMENT_ERROR (dec, STREAM, DECODE, (NULL),
+        ("Picture has unknown or unsupported colourspace"));
+    ret = GST_FLOW_ERROR;
+    goto done;
+  }
+invalid_yuvrgbgrayscale:
+  {
+    GST_ELEMENT_ERROR (dec, STREAM, DECODE, (NULL),
+        ("Picture is corrupt or unhandled YUV/RGB/grayscale layout"));
+    ret = GST_FLOW_ERROR;
+    goto done;
+  }
 }
 
 static gboolean
@@ -1342,6 +1563,12 @@ gst_jpeg_dec_sink_event (GstPad * pad, GstEvent * event)
       GST_DEBUG_OBJECT (dec, "Aborting decompress");
       jpeg_abort_decompress (&dec->cinfo);
       gst_segment_init (&dec->segment, GST_FORMAT_UNDEFINED);
+      gst_adapter_clear (dec->adapter);
+      g_free (dec->cur_buf);
+      dec->cur_buf = NULL;
+      dec->parse_offset = 0;
+      dec->parse_entropy_len = 0;
+      dec->parse_resync = FALSE;
       gst_jpeg_dec_reset_qos (dec);
       break;
     case GST_EVENT_NEWSEGMENT:{
@@ -1425,9 +1652,14 @@ gst_jpeg_dec_change_state (GstElement * element, GstStateChange transition)
       dec->caps_framerate_numerator = dec->caps_framerate_denominator = 0;
       dec->caps_width = -1;
       dec->caps_height = -1;
+      dec->clrspc = -1;
       dec->packetized = FALSE;
       dec->next_ts = 0;
       dec->discont = TRUE;
+      dec->parse_offset = 0;
+      dec->parse_entropy_len = 0;
+      dec->parse_resync = FALSE;
+      dec->cur_buf = NULL;
       gst_segment_init (&dec->segment, GST_FORMAT_UNDEFINED);
       gst_jpeg_dec_reset_qos (dec);
     default:
@@ -1440,10 +1672,9 @@ gst_jpeg_dec_change_state (GstElement * element, GstStateChange transition)
 
   switch (transition) {
     case GST_STATE_CHANGE_PAUSED_TO_READY:
-      if (dec->tempbuf) {
-        gst_buffer_unref (dec->tempbuf);
-        dec->tempbuf = NULL;
-      }
+      gst_adapter_clear (dec->adapter);
+      g_free (dec->cur_buf);
+      dec->cur_buf = NULL;
       gst_jpeg_dec_free_buffers (dec);
       break;
     default:
