@@ -42,6 +42,7 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <string.h>
+#include <stdlib.h>
 
 #include <libavc1394/avc1394.h>
 #include <libavc1394/avc1394_vcr.h>
@@ -55,6 +56,7 @@
 
 #include "gstdv1394src.h"
 #include "gst1394probe.h"
+#include "gst1394clock.h"
 
 
 #define CONTROL_STOP            'S'     /* stop the select call */
@@ -111,14 +113,6 @@ enum
   PROP_DEVICE_NAME
 };
 
-static const GstElementDetails gst_dv1394src_details =
-GST_ELEMENT_DETAILS ("Firewire (1394) DV video source",
-    "Source/Video",
-    "Source for DV video data from firewire port",
-    "Erik Walthinsen <omega@temple-baptist.com>\n"
-    "Daniel Fischer <dan@f3c.com>\n" "Wim Taymans <wim@fluendo.com>\n"
-    "Zaheer Abbas Merali <zaheerabbas at merali dot org>");
-
 static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
@@ -135,18 +129,17 @@ static void gst_dv1394src_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 static void gst_dv1394src_dispose (GObject * object);
 
+static GstClock *gst_dv1394src_provide_clock (GstElement * element);
+static GstStateChangeReturn gst_dv1394_src_change_state (GstElement * element,
+    GstStateChange transition);
+
 static gboolean gst_dv1394src_start (GstBaseSrc * bsrc);
 static gboolean gst_dv1394src_stop (GstBaseSrc * bsrc);
 static gboolean gst_dv1394src_unlock (GstBaseSrc * bsrc);
 
 static GstFlowReturn gst_dv1394src_create (GstPushSrc * psrc, GstBuffer ** buf);
 
-static gboolean gst_dv1394src_convert (GstPad * pad,
-    GstFormat src_format, gint64 src_value,
-    GstFormat * dest_format, gint64 * dest_value);
-
-static const GstQueryType *gst_dv1394src_get_query_types (GstPad * pad);
-static gboolean gst_dv1394src_query (GstPad * pad, GstQuery * query);
+static gboolean gst_dv1394src_query (GstBaseSrc * src, GstQuery * query);
 static void gst_dv1394src_update_device_name (GstDV1394Src * src);
 
 static void
@@ -180,17 +173,24 @@ gst_dv1394src_base_init (gpointer g_class)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&src_factory));
 
-  gst_element_class_set_details (element_class, &gst_dv1394src_details);
+  gst_element_class_set_details_simple (element_class,
+      "Firewire (1394) DV video source", "Source/Video",
+      "Source for DV video data from firewire port",
+      "Erik Walthinsen <omega@temple-baptist.com>, "
+      "Daniel Fischer <dan@f3c.com>, " "Wim Taymans <wim@fluendo.com>, "
+      "Zaheer Abbas Merali <zaheerabbas at merali dot org>");
 }
 
 static void
 gst_dv1394src_class_init (GstDV1394SrcClass * klass)
 {
   GObjectClass *gobject_class;
+  GstElementClass *gstelement_class;
   GstBaseSrcClass *gstbasesrc_class;
   GstPushSrcClass *gstpushsrc_class;
 
   gobject_class = (GObjectClass *) klass;
+  gstelement_class = (GstElementClass *) klass;
   gstbasesrc_class = (GstBaseSrcClass *) klass;
   gstpushsrc_class = (GstPushSrcClass *) klass;
 
@@ -198,6 +198,9 @@ gst_dv1394src_class_init (GstDV1394SrcClass * klass)
   gobject_class->get_property = gst_dv1394src_get_property;
   gobject_class->dispose = gst_dv1394src_dispose;
 
+  gstelement_class->provide_clock = gst_dv1394src_provide_clock;
+  gstelement_class->change_state = gst_dv1394_src_change_state;
+
   gst_dv1394src_signals[SIGNAL_FRAME_DROPPED] =
       g_signal_new ("frame-dropped", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstDV1394SrcClass, frame_dropped),
@@ -243,6 +246,7 @@ gst_dv1394src_class_init (GstDV1394SrcClass * klass)
   gstbasesrc_class->start = gst_dv1394src_start;
   gstbasesrc_class->stop = gst_dv1394src_stop;
   gstbasesrc_class->unlock = gst_dv1394src_unlock;
+  gstbasesrc_class->query = gst_dv1394src_query;
 
   gstpushsrc_class->create = gst_dv1394src_create;
 }
@@ -253,11 +257,10 @@ gst_dv1394src_init (GstDV1394Src * dv1394src, GstDV1394SrcClass * klass)
   GstPad *srcpad = GST_BASE_SRC_PAD (dv1394src);
 
   gst_base_src_set_live (GST_BASE_SRC (dv1394src), TRUE);
+  gst_base_src_set_format (GST_BASE_SRC (dv1394src), GST_FORMAT_TIME);
+  gst_base_src_set_do_timestamp (GST_BASE_SRC (dv1394src), TRUE);
   gst_pad_use_fixed_caps (srcpad);
 
-  gst_pad_set_query_function (srcpad, gst_dv1394src_query);
-  gst_pad_set_query_type_function (srcpad, gst_dv1394src_get_query_types);
-
   dv1394src->port = DEFAULT_PORT;
   dv1394src->channel = DEFAULT_CHANNEL;
 
@@ -278,6 +281,8 @@ gst_dv1394src_init (GstDV1394Src * dv1394src, GstDV1394SrcClass * klass)
   dv1394src->buf = NULL;
   dv1394src->frame = NULL;
   dv1394src->frame_sequence = 0;
+
+  dv1394src->provided_clock = gst_1394_clock_new ("dv1394clock");
 }
 
 static void
@@ -285,6 +290,10 @@ gst_dv1394src_dispose (GObject * object)
 {
   GstDV1394Src *src = GST_DV1394SRC (object);
 
+  if (src->provided_clock) {
+    g_object_unref (src->provided_clock);
+  }
+
   g_free (src->uri);
   src->uri = NULL;
 
@@ -367,6 +376,47 @@ gst_dv1394src_get_property (GObject * object, guint prop_id, GValue * value,
   }
 }
 
+static GstClock *
+gst_dv1394src_provide_clock (GstElement * element)
+{
+  GstDV1394Src *dv1394src = GST_DV1394SRC (element);
+
+  return GST_CLOCK_CAST (gst_object_ref (dv1394src->provided_clock));
+}
+
+static GstStateChangeReturn
+gst_dv1394_src_change_state (GstElement * element, GstStateChange transition)
+{
+  GstStateChangeReturn ret = GST_STATE_CHANGE_SUCCESS;
+  GstDV1394Src *src = GST_DV1394SRC (element);
+
+  switch (transition) {
+    case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
+      gst_element_post_message (element,
+          gst_message_new_clock_lost (GST_OBJECT_CAST (element),
+              GST_CLOCK_CAST (src->provided_clock)));
+      break;
+    default:
+      break;
+  }
+
+  ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
+  if (ret == GST_STATE_CHANGE_FAILURE)
+    return ret;
+
+  switch (transition) {
+    case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
+      gst_element_post_message (element,
+          gst_message_new_clock_provide (GST_OBJECT_CAST (element),
+              GST_CLOCK_CAST (src->provided_clock), TRUE));
+      break;
+    default:
+      break;
+  }
+
+  return ret;
+}
+
 #ifdef HAVE_LIBIEC61883
 static GstDV1394Src *
 gst_dv1394src_from_raw1394handle (raw1394handle_t handle)
@@ -391,7 +441,7 @@ gst_dv1394src_iec61883_receive (unsigned char *data, int len,
 {
   GstDV1394Src *dv1394src = GST_DV1394SRC (cbdata);
 
-  if (!GST_PAD_CAPS (GST_BASE_SRC_PAD (dv1394src))) {
+  if (G_UNLIKELY (!GST_PAD_CAPS (GST_BASE_SRC_PAD (dv1394src)))) {
     GstCaps *caps;
     unsigned char *p = data;
 
@@ -417,26 +467,17 @@ gst_dv1394src_iec61883_receive (unsigned char *data, int len,
     gst_pad_set_caps (GST_BASE_SRC_PAD (dv1394src), caps);
     gst_caps_unref (caps);
   }
+
   dv1394src->frame = NULL;
-  if ((dv1394src->frame_sequence + 1) % (dv1394src->skip +
-          dv1394src->consecutive) < dv1394src->consecutive) {
+  if (G_LIKELY ((dv1394src->frame_sequence + 1) % (dv1394src->skip +
+              dv1394src->consecutive) < dv1394src->consecutive)) {
     if (complete && len == dv1394src->frame_size) {
-      gint64 i64;
       guint8 *bufdata;
       GstBuffer *buf;
-      GstFormat format;
 
       buf = gst_buffer_new_and_alloc (dv1394src->frame_size);
 
-      /* fill in offset, duration, timestamp */
       GST_BUFFER_OFFSET (buf) = dv1394src->frame_sequence;
-      format = GST_FORMAT_TIME;
-      gst_dv1394src_convert (GST_BASE_SRC_PAD (dv1394src), GST_FORMAT_DEFAULT,
-          GST_BUFFER_OFFSET (buf), &format, &i64);
-      GST_BUFFER_TIMESTAMP (buf) = i64;
-      gst_dv1394src_convert (GST_BASE_SRC_PAD (dv1394src), GST_FORMAT_DEFAULT,
-          1, &format, &i64);
-      GST_BUFFER_DURATION (buf) = i64;
       bufdata = GST_BUFFER_DATA (buf);
       memcpy (bufdata, data, len);
       dv1394src->buf = buf;
@@ -508,7 +549,6 @@ gst_dv1394src_iso_receive (raw1394handle_t handle, int channel, size_t len,
       }
       if ((dv1394src->frame_sequence + 1) % (dv1394src->skip +
               dv1394src->consecutive) < dv1394src->consecutive) {
-        GstFormat format;
         GstBuffer *buf;
         gint64 i64;
 
@@ -516,14 +556,6 @@ gst_dv1394src_iso_receive (raw1394handle_t handle, int channel, size_t len,
 
         /* fill in offset, duration, timestamp */
         GST_BUFFER_OFFSET (buf) = dv1394src->frame_sequence;
-        format = GST_FORMAT_TIME;
-        gst_dv1394src_convert (GST_BASE_SRC_PAD (dv1394src), GST_FORMAT_DEFAULT,
-            GST_BUFFER_OFFSET (buf), &format, &i64);
-        GST_BUFFER_TIMESTAMP (buf) = i64;
-        gst_dv1394src_convert (GST_BASE_SRC_PAD (dv1394src), GST_FORMAT_DEFAULT,
-            1, &format, &i64);
-        GST_BUFFER_DURATION (buf) = i64;
-
         dv1394src->frame = buf;
       }
       dv1394src->frame_sequence++;
@@ -641,7 +673,7 @@ gst_dv1394src_create (GstPushSrc * psrc, GstBuffer ** buf)
   pollfds[1].fd = READ_SOCKET (dv1394src);
   pollfds[1].events = POLLIN | POLLERR | POLLHUP | POLLPRI;
 
-  if (dv1394src->buf) {
+  if (G_UNLIKELY (dv1394src->buf)) {
     /* maybe we had an error before, and there's a stale buffer? */
     gst_buffer_unref (dv1394src->buf);
     dv1394src->buf = NULL;
@@ -650,12 +682,13 @@ gst_dv1394src_create (GstPushSrc * psrc, GstBuffer ** buf)
   while (TRUE) {
     int res = poll (pollfds, 2, -1);
 
-    if (res < 0) {
+    if (G_UNLIKELY (res < 0)) {
       if (errno == EAGAIN || errno == EINTR)
         continue;
       else
         goto error_while_polling;
     }
+
     if (G_UNLIKELY (pollfds[1].revents)) {
       char command;
 
@@ -710,7 +743,6 @@ gst_dv1394src_discover_avc_node (GstDV1394Src * src)
   /* loop over all our ports */
   for (; j < m && node == -1; j++) {
     raw1394handle_t handle;
-    gint n_ports;
     struct raw1394_portinfo pinf[16];
 
     /* open the port */
@@ -719,7 +751,7 @@ gst_dv1394src_discover_avc_node (GstDV1394Src * src)
       GST_WARNING ("raw1394 - failed to get handle: %s.\n", strerror (errno));
       continue;
     }
-    if ((n_ports = raw1394_get_port_info (handle, pinf, 16)) < 0) {
+    if (raw1394_get_port_info (handle, pinf, 16) < 0) {
       GST_WARNING ("raw1394 - failed to get port info: %s.\n",
           strerror (errno));
       goto next;
@@ -855,6 +887,8 @@ gst_dv1394src_start (GstBaseSrc * bsrc)
     }
   }
 
+  gst_1394_clock_set_handle (src->provided_clock, src->handle);
+
   return TRUE;
 
 socket_pair:
@@ -949,6 +983,8 @@ gst_dv1394src_stop (GstBaseSrc * bsrc)
     }
   }
 
+  gst_1394_clock_unset_handle (src->provided_clock);
+
   raw1394_destroy_handle (src->handle);
 
   return TRUE;
@@ -965,122 +1001,22 @@ gst_dv1394src_unlock (GstBaseSrc * bsrc)
 }
 
 static gboolean
-gst_dv1394src_convert (GstPad * pad,
-    GstFormat src_format, gint64 src_value,
-    GstFormat * dest_format, gint64 * dest_value)
+gst_dv1394src_query (GstBaseSrc * basesrc, GstQuery * query)
 {
-  GstDV1394Src *src;
-
-  src = GST_DV1394SRC (gst_pad_get_parent (pad));
-
-  switch (src_format) {
-    case GST_FORMAT_TIME:
-      switch (*dest_format) {
-        case GST_FORMAT_BYTES:
-          src_value *= src->frame_size;
-        case GST_FORMAT_DEFAULT:
-          *dest_value =
-              gst_util_uint64_scale_int (src_value, src->frame_rate,
-              GST_SECOND);
-          break;
-        default:
-          goto not_supported;
-      }
-      break;
-    case GST_FORMAT_BYTES:
-      src_value /= src->frame_size;
-    case GST_FORMAT_DEFAULT:
-      switch (*dest_format) {
-        case GST_FORMAT_BYTES:
-          *dest_value = src_value * src->frame_size;
-          break;
-        case GST_FORMAT_TIME:
-          if (src->frame_rate != 0)
-            *dest_value =
-                gst_util_uint64_scale_int (src_value, GST_SECOND,
-                src->frame_rate);
-          else
-            goto not_supported;
-          break;
-        default:
-          goto not_supported;
-      }
-      break;
-    default:
-      goto not_supported;
-  }
-
-  gst_object_unref (src);
-  return TRUE;
-
-not_supported:
-  {
-    GST_DEBUG_OBJECT (src, "unsupported conversion");
-    gst_object_unref (src);
-    return FALSE;
-  }
-}
-
-static const GstQueryType *
-gst_dv1394src_get_query_types (GstPad * pad)
-{
-  static const GstQueryType src_query_types[] = {
-    GST_QUERY_CONVERT,
-    GST_QUERY_POSITION,
-    0
-  };
-
-  return src_query_types;
-}
-
-static gboolean
-gst_dv1394src_query (GstPad * pad, GstQuery * query)
-{
-  gboolean res = TRUE;
-  GstDV1394Src *src;
-
-  src = GST_DV1394SRC (gst_pad_get_parent (pad));
-
   switch (GST_QUERY_TYPE (query)) {
-    case GST_QUERY_POSITION:
+    case GST_QUERY_LATENCY:
     {
-      GstFormat format;
-      gint64 current;
-
-      gst_query_parse_position (query, &format, NULL);
-
-      /* bring our current frame to the requested format */
-      res = gst_pad_query_convert (pad,
-          GST_FORMAT_DEFAULT, src->frame_sequence, &format, &current);
-
-      gst_query_set_position (query, format, current);
-      break;
+      gst_query_set_latency (query, TRUE, GST_SECOND / 25, GST_CLOCK_TIME_NONE);
     }
-    case GST_QUERY_CONVERT:
-    {
-      GstFormat src_fmt, dest_fmt;
-      gint64 src_val, dest_val;
-
-      gst_query_parse_convert (query, &src_fmt, &src_val, &dest_fmt, &dest_val);
-      if (!(res =
-              gst_dv1394src_convert (pad, src_fmt, src_val, &dest_fmt,
-                  &dest_val)))
-        goto not_supported;
-      gst_query_set_convert (query, src_fmt, src_val, dest_fmt, dest_val);
       break;
-    }
     default:
       goto not_supported;
   }
 
-  gst_object_unref (src);
   return TRUE;
 
 not_supported:
-  {
-    gst_object_unref (src);
-    return FALSE;
-  }
+  return GST_BASE_SRC_CLASS (parent_class)->query (basesrc, query);
 }
 
 static void
@@ -1146,7 +1082,7 @@ gst_dv1394src_uri_get_type (void)
 static gchar **
 gst_dv1394src_uri_get_protocols (void)
 {
-  static gchar *protocols[] = { "dv", NULL };
+  static gchar *protocols[] = { (char *) "dv", NULL };
 
   return protocols;
 }
