@@ -91,13 +91,11 @@ gst_auto_audio_sink_class_init (GstAutoAudioSinkClass * klass)
   gobject_class = G_OBJECT_CLASS (klass);
   eklass = GST_ELEMENT_CLASS (klass);
 
-  gobject_class->dispose =
-      (GObjectFinalizeFunc) GST_DEBUG_FUNCPTR (gst_auto_audio_sink_dispose);
+  gobject_class->dispose = (GObjectFinalizeFunc) gst_auto_audio_sink_dispose;
+  gobject_class->set_property = gst_auto_audio_sink_set_property;
+  gobject_class->get_property = gst_auto_audio_sink_get_property;
+
   eklass->change_state = GST_DEBUG_FUNCPTR (gst_auto_audio_sink_change_state);
-  gobject_class->set_property =
-      GST_DEBUG_FUNCPTR (gst_auto_audio_sink_set_property);
-  gobject_class->get_property =
-      GST_DEBUG_FUNCPTR (gst_auto_audio_sink_get_property);
 
   /**
    * GstAutoAudioSink:filter-caps
@@ -135,6 +133,8 @@ gst_auto_audio_sink_clear_kid (GstAutoAudioSink * sink)
     gst_element_set_state (sink->kid, GST_STATE_NULL);
     gst_bin_remove (GST_BIN (sink), sink->kid);
     sink->kid = NULL;
+    /* Don't lose the SINK flag */
+    GST_OBJECT_FLAG_SET (sink, GST_ELEMENT_IS_SINK);
   }
 }
 
