@@ -56,11 +56,6 @@ typedef struct {
   guint64        total;   /* total bytes before */
 } GstAviIndexEntry;
 
-#define GST_AVI_KEYFRAME 1
-#define ENTRY_IS_KEYFRAME(e) ((e)->flags == GST_AVI_KEYFRAME)
-#define ENTRY_SET_KEYFRAME(e) ((e)->flags = GST_AVI_KEYFRAME)
-#define ENTRY_UNSET_KEYFRAME(e) ((e)->flags = 0)
-
 typedef struct {
   /* index of this streamcontext */
   guint          num;
@@ -129,6 +124,7 @@ typedef enum {
   GST_AVI_DEMUX_START,
   GST_AVI_DEMUX_HEADER,
   GST_AVI_DEMUX_MOVI,
+  GST_AVI_DEMUX_SEEK,
 } GstAviDemuxState;
 
 typedef enum {
@@ -174,13 +170,14 @@ typedef struct _GstAviDemux {
 
   /* some stream info for length */
   gst_riff_avih *avih;
+  GstClockTime   duration;
 
   /* segment in TIME */
   GstSegment     segment;
   gboolean       segment_running;
 
   /* pending tags/events */
-  GstEvent      *seek_event;
+  GstEvent      *seg_event;
   GstTagList	*globaltags;
   gboolean	 got_tags;
 
@@ -188,6 +185,17 @@ typedef struct _GstAviDemux {
   GstIndex      *element_index;
   gint           index_id;
   gboolean       seekable;
+
+  guint64        first_movi_offset;
+  guint64        idx1_offset; /* offset in file of list/chunk after movi */
+  GstEvent      *seek_event;
+
+  gboolean       building_index;
+  guint          odml_stream;
+  guint          odml_subidx;
+  guint64       *odml_subidxs;
+
+  guint64        seek_kf_offset; /* offset of the keyframe to which we want to seek */
 } GstAviDemux;
 
 typedef struct _GstAviDemuxClass {
