@@ -26,6 +26,13 @@
 
 #include "ebml-ids.h"
 
+/*
+ * EBML DocType.
+ */
+
+#define GST_MATROSKA_DOCTYPE_MATROSKA              "matroska"
+#define GST_MATROSKA_DOCTYPE_WEBM                  "webm"
+
 /*
  * Matroska element IDs. max. 32-bit.
  */
@@ -337,6 +344,7 @@
 #define GST_MATROSKA_CODEC_ID_VIDEO_QUICKTIME    "V_QUICKTIME"
 #define GST_MATROSKA_CODEC_ID_VIDEO_SNOW         "V_SNOW"
 #define GST_MATROSKA_CODEC_ID_VIDEO_DIRAC        "V_DIRAC"
+#define GST_MATROSKA_CODEC_ID_VIDEO_VP8          "V_VP8"
 
 #define GST_MATROSKA_CODEC_ID_AUDIO_MPEG1_L1       "A_MPEG/L1"
 #define GST_MATROSKA_CODEC_ID_AUDIO_MPEG1_L2       "A_MPEG/L2"
@@ -466,6 +474,10 @@ struct _GstMatroskaTrackContext {
   GstCaps      *caps;
   guint         index;
   GstFlowReturn last_flow;
+  /* reverse playback */
+  GstClockTime  from_time;
+  gint64                   from_offset;
+  gint64                   to_offset;
 
   GArray       *index_table;
 
@@ -521,6 +533,9 @@ struct _GstMatroskaTrackContext {
   /* A GArray of GstMatroskaTrackEncoding structures which contain the
    * encoding (compression/encryption) settings for this track, if any */
   GArray       *encodings;
+
+  /* Whether the stream is EOS */
+  gboolean      eos;
 };
 
 typedef struct _GstMatroskaTrackVideoContext {
@@ -532,6 +547,9 @@ typedef struct _GstMatroskaTrackVideoContext {
   GstMatroskaAspectRatioMode asr_mode;
   guint32       fourcc;
 
+  /* QoS */
+  GstClockTime  earliest_time;
+
   GstBuffer     *dirac_unit;
 } GstMatroskaTrackVideoContext;
 
