@@ -28,14 +28,6 @@
 #include <gst/rtp/gstrtpbuffer.h>
 #include "gstrtpsirendepay.h"
 
-/* elementfactory information */
-static const GstElementDetails gst_rtp_siren_depay_details =
-GST_ELEMENT_DETAILS ("RTP Siren packet depayloader",
-    "Codec/Depayloader/Network",
-    "Extracts Siren audio from RTP packets",
-    "Philippe Kalaf <philippe.kalaf@collabora.co.uk>");
-
-
 static GstStaticPadTemplate gst_rtp_siren_depay_sink_template =
 GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
@@ -71,7 +63,10 @@ gst_rtp_siren_depay_base_init (gpointer klass)
       gst_static_pad_template_get (&gst_rtp_siren_depay_src_template));
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_rtp_siren_depay_sink_template));
-  gst_element_class_set_details (element_class, &gst_rtp_siren_depay_details);
+  gst_element_class_set_details_simple (element_class,
+      "RTP Siren packet depayloader", "Codec/Depayloader/Network",
+      "Extracts Siren audio from RTP packets",
+      "Philippe Kalaf <philippe.kalaf@collabora.co.uk>");
 }
 
 static void
