@@ -121,13 +121,6 @@
 GST_DEBUG_CATEGORY_STATIC (gst_rtp_session_debug);
 #define GST_CAT_DEFAULT gst_rtp_session_debug
 
-/* elementfactory information */
-static const GstElementDetails rtpsession_details =
-GST_ELEMENT_DETAILS ("RTP Session",
-    "Filter/Network/RTP",
-    "Implement an RTP session",
-    "Wim Taymans <wim.taymans@gmail.com>");
-
 /* sink pads */
 static GstStaticPadTemplate rtpsession_recv_rtp_sink_template =
 GST_STATIC_PAD_TEMPLATE ("recv_rtp_sink",
@@ -200,6 +193,8 @@ enum
 #define DEFAULT_NTP_NS_BASE          0
 #define DEFAULT_BANDWIDTH            RTP_STATS_BANDWIDTH
 #define DEFAULT_RTCP_FRACTION        RTP_STATS_RTCP_BANDWIDTH
+#define DEFAULT_RTCP_RR_BANDWIDTH    -1
+#define DEFAULT_RTCP_RS_BANDWIDTH    -1
 #define DEFAULT_SDES                 NULL
 #define DEFAULT_NUM_SOURCES          0
 #define DEFAULT_NUM_ACTIVE_SOURCES   0
@@ -210,6 +205,8 @@ enum
   PROP_NTP_NS_BASE,
   PROP_BANDWIDTH,
   PROP_RTCP_FRACTION,
+  PROP_RTCP_RR_BANDWIDTH,
+  PROP_RTCP_RS_BANDWIDTH,
   PROP_SDES,
   PROP_NUM_SOURCES,
   PROP_NUM_ACTIVE_SOURCES,
@@ -382,7 +379,9 @@ gst_rtp_session_base_init (gpointer klass)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&rtpsession_send_rtcp_src_template));
 
-  gst_element_class_set_details (element_class, &rtpsession_details);
+  gst_element_class_set_details_simple (element_class, "RTP Session",
+      "Filter/Network/RTP",
+      "Implement an RTP session", "Wim Taymans <wim.taymans@gmail.com>");
 }
 
 static void
@@ -535,14 +534,24 @@ gst_rtp_session_class_init (GstRtpSessionClass * klass)
 
   g_object_class_install_property (gobject_class, PROP_BANDWIDTH,
       g_param_spec_double ("bandwidth", "Bandwidth",
-          "The bandwidth of the session",
+          "The bandwidth of the session in bytes per second",
           0.0, G_MAXDOUBLE, DEFAULT_BANDWIDTH, G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class, PROP_RTCP_FRACTION,
       g_param_spec_double ("rtcp-fraction", "RTCP Fraction",
-          "The fraction of the bandwidth used for RTCP",
+          "The RTCP bandwidth of the session in bytes per second",
           0.0, G_MAXDOUBLE, DEFAULT_RTCP_FRACTION, G_PARAM_READWRITE));
 
+  g_object_class_install_property (gobject_class, PROP_RTCP_RR_BANDWIDTH,
+      g_param_spec_int ("rtcp-rr-bandwidth", "RTCP RR bandwidth",
+          "The RTCP bandwidth used for receivers in bytes per second (-1 = default)",
+          -1, G_MAXINT, DEFAULT_RTCP_RR_BANDWIDTH, G_PARAM_READWRITE));
+
+  g_object_class_install_property (gobject_class, PROP_RTCP_RS_BANDWIDTH,
+      g_param_spec_int ("rtcp-rs-bandwidth", "RTCP RS bandwidth",
+          "The RTCP bandwidth used for senders in bytes per second (-1 = default)",
+          -1, G_MAXINT, DEFAULT_RTCP_RS_BANDWIDTH, G_PARAM_READWRITE));
+
   g_object_class_install_property (gobject_class, PROP_SDES,
       g_param_spec_boxed ("sdes", "SDES",
           "The SDES items of this session",
@@ -657,10 +666,18 @@ gst_rtp_session_set_property (GObject * object, guint prop_id,
       GST_OBJECT_UNLOCK (rtpsession);
       break;
     case PROP_BANDWIDTH:
-      rtp_session_set_bandwidth (priv->session, g_value_get_double (value));
+      g_object_set_property (G_OBJECT (priv->session), "bandwidth", value);
       break;
     case PROP_RTCP_FRACTION:
-      rtp_session_set_rtcp_fraction (priv->session, g_value_get_double (value));
+      g_object_set_property (G_OBJECT (priv->session), "rtcp-fraction", value);
+      break;
+    case PROP_RTCP_RR_BANDWIDTH:
+      g_object_set_property (G_OBJECT (priv->session), "rtcp-rr-bandwidth",
+          value);
+      break;
+    case PROP_RTCP_RS_BANDWIDTH:
+      g_object_set_property (G_OBJECT (priv->session), "rtcp-rs-bandwidth",
+          value);
       break;
     case PROP_SDES:
       rtp_session_set_sdes_struct (priv->session, g_value_get_boxed (value));
@@ -688,10 +705,18 @@ gst_rtp_session_get_property (GObject * object, guint prop_id,
       GST_OBJECT_UNLOCK (rtpsession);
       break;
     case PROP_BANDWIDTH:
-      g_value_set_double (value, rtp_session_get_bandwidth (priv->session));
+      g_object_get_property (G_OBJECT (priv->session), "bandwidth", value);
       break;
     case PROP_RTCP_FRACTION:
-      g_value_set_double (value, rtp_session_get_rtcp_fraction (priv->session));
+      g_object_get_property (G_OBJECT (priv->session), "rtcp-fraction", value);
+      break;
+    case PROP_RTCP_RR_BANDWIDTH:
+      g_object_get_property (G_OBJECT (priv->session), "rtcp-rr-bandwidth",
+          value);
+      break;
+    case PROP_RTCP_RS_BANDWIDTH:
+      g_object_get_property (G_OBJECT (priv->session), "rtcp-rs-bandwidth",
+          value);
       break;
     case PROP_SDES:
       g_value_take_boxed (value, rtp_session_get_sdes_struct (priv->session));
@@ -757,19 +782,23 @@ rtcp_thread (GstRtpSession * rtpsession)
   GstClockTime next_timeout;
   guint64 ntpnstime;
   GstClockTime running_time;
+  RTPSession *session;
+  GstClock *sysclock;
 
   GST_DEBUG_OBJECT (rtpsession, "entering RTCP thread");
 
   GST_RTP_SESSION_LOCK (rtpsession);
 
-  current_time = gst_clock_get_time (rtpsession->priv->sysclock);
+  sysclock = rtpsession->priv->sysclock;
+  current_time = gst_clock_get_time (sysclock);
+
+  session = rtpsession->priv->session;
 
   while (!rtpsession->priv->stop_thread) {
     GstClockReturn res;
 
     /* get initial estimate */
-    next_timeout =
-        rtp_session_next_timeout (rtpsession->priv->session, current_time);
+    next_timeout = rtp_session_next_timeout (session, current_time);
 
     GST_DEBUG_OBJECT (rtpsession, "next check time %" GST_TIME_FORMAT,
         GST_TIME_ARGS (next_timeout));
@@ -779,7 +808,7 @@ rtcp_thread (GstRtpSession * rtpsession)
       break;
 
     id = rtpsession->priv->id =
-        gst_clock_new_single_shot_id (rtpsession->priv->sysclock, next_timeout);
+        gst_clock_new_single_shot_id (sysclock, next_timeout);
     GST_RTP_SESSION_UNLOCK (rtpsession);
 
     res = gst_clock_id_wait (id, NULL);
@@ -792,7 +821,7 @@ rtcp_thread (GstRtpSession * rtpsession)
       break;
 
     /* update current time */
-    current_time = gst_clock_get_time (rtpsession->priv->sysclock);
+    current_time = gst_clock_get_time (sysclock);
 
     /* get current NTP time */
     get_current_times (rtpsession, &running_time, &ntpnstime);
@@ -804,8 +833,7 @@ rtcp_thread (GstRtpSession * rtpsession)
 
     /* perform actions, we ignore result. Release lock because it might push. */
     GST_RTP_SESSION_UNLOCK (rtpsession);
-    rtp_session_on_timeout (rtpsession->priv->session, current_time, ntpnstime,
-        running_time);
+    rtp_session_on_timeout (session, current_time, ntpnstime, running_time);
     GST_RTP_SESSION_LOCK (rtpsession);
   }
   /* mark the thread as stopped now */
