@@ -54,7 +54,6 @@
 
 #include <gst/interfaces/mixer.h>
 #include <gst/gst-i18n-plugin.h>
-#include "gst/glib-compat-private.h"
 
 #include <glib/gprintf.h>
 
@@ -141,7 +140,7 @@ gst_oss4_mixer_class_init (GstOss4MixerClass * klass)
       g_param_spec_string ("device", "Device",
           "OSS mixer device (e.g. /dev/oss/hdaudio0/mix0 or /dev/mixerN) "
           "(NULL = use first mixer device found)", DEFAULT_DEVICE,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          G_PARAM_READWRITE));
 
   /**
    * GstOss4Mixer:device-name
@@ -153,7 +152,7 @@ gst_oss4_mixer_class_init (GstOss4MixerClass * klass)
   g_object_class_install_property (gobject_class, PROP_DEVICE_NAME,
       g_param_spec_string ("device-name", "Device name",
           "Human-readable name of the sound device", DEFAULT_DEVICE_NAME,
-          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+          G_PARAM_READABLE));
 
   element_class->change_state = GST_DEBUG_FUNCPTR (gst_oss4_mixer_change_state);
 }
@@ -542,13 +541,8 @@ gst_oss4_mixer_start_watch_task (GstOss4Mixer * mixer)
   mixer->watch_cond = g_cond_new ();
   mixer->watch_shutdown = FALSE;
 
-#if !GLIB_CHECK_VERSION (2, 31, 0)
   mixer->watch_thread = g_thread_create (gst_oss4_mixer_watch_thread,
       gst_object_ref (mixer), TRUE, &err);
-#else
-  mixer->watch_thread = g_thread_try_new ("oss4-mixer-thread",
-      gst_oss4_mixer_watch_thread, gst_object_ref (mixer), &err);
-#endif
 
   if (mixer->watch_thread == NULL) {
     GST_ERROR_OBJECT (mixer, "Could not create watch thread: %s", err->message);
