@@ -53,13 +53,6 @@
 
 #include <string.h>
 
-
-static const GstElementDetails gst_v4l2sink_details =
-GST_ELEMENT_DETAILS ("Video (video4linux2) Sink",
-    "Sink/Video",
-    "Displays frames on a video4linux2 device",
-    "Rob Clark <rob@ti.com>,");
-
 GST_DEBUG_CATEGORY (v4l2sink_debug);
 #define GST_CAT_DEFAULT v4l2sink_debug
 
@@ -119,7 +112,7 @@ gst_v4l2sink_interface_init (GstImplementsInterfaceClass * klass)
   klass->supported = gst_v4l2sink_iface_supported;
 }
 
-void
+static void
 gst_v4l2sink_init_interfaces (GType type)
 {
   static const GInterfaceInfo v4l2iface_info = {
@@ -201,7 +194,9 @@ gst_v4l2sink_base_init (gpointer g_class)
 
   GST_DEBUG_CATEGORY_INIT (v4l2sink_debug, "v4l2sink", 0, "V4L2 sink element");
 
-  gst_element_class_set_details (gstelement_class, &gst_v4l2sink_details);
+  gst_element_class_set_details_simple (gstelement_class,
+      "Video (video4linux2) Sink", "Sink/Video",
+      "Displays frames on a video4linux2 device", "Rob Clark <rob@ti.com>,");
 
   gst_element_class_add_pad_template
       (gstelement_class,
@@ -233,23 +228,23 @@ gst_v4l2sink_class_init (GstV4l2SinkClass * klass)
       g_param_spec_uint ("queue-size", "Queue size",
           "Number of buffers to be enqueud in the driver in streaming mode",
           GST_V4L2_MIN_BUFFERS, GST_V4L2_MAX_BUFFERS, PROP_DEF_QUEUE_SIZE,
-          G_PARAM_READWRITE));
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_OVERLAY_TOP,
       g_param_spec_int ("overlay-top", "Overlay top",
           "The topmost (y) coordinate of the video overlay; top left corner of screen is 0,0",
-          0x80000000, 0x7fffffff, 0, G_PARAM_READWRITE));
+          G_MININT, G_MAXINT, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_OVERLAY_LEFT,
       g_param_spec_int ("overlay-left", "Overlay left",
           "The leftmost (x) coordinate of the video overlay; top left corner of screen is 0,0",
-          0x80000000, 0x7fffffff, 0, G_PARAM_READWRITE));
+          G_MININT, G_MAXINT, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_OVERLAY_WIDTH,
       g_param_spec_uint ("overlay-width", "Overlay width",
           "The width of the video overlay; default is equal to negotiated image width",
-          0, 0xffffffff, 0, G_PARAM_READWRITE));
+          0, G_MAXUINT, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_OVERLAY_HEIGHT,
       g_param_spec_uint ("overlay-height", "Overlay height",
           "The height of the video overlay; default is equal to negotiated image height",
-          0, 0xffffffff, 0, G_PARAM_READWRITE));
+          0, G_MAXUINT, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   basesink_class->get_caps = GST_DEBUG_FUNCPTR (gst_v4l2sink_get_caps);
   basesink_class->set_caps = GST_DEBUG_FUNCPTR (gst_v4l2sink_set_caps);
@@ -466,6 +461,8 @@ gst_v4l2sink_change_state (GstElement * element, GstStateChange transition)
       }
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
+      gst_v4l2_buffer_pool_destroy (v4l2sink->pool);
+      v4l2sink->pool = NULL;
       /* close the device */
       if (!gst_v4l2_object_stop (v4l2sink->v4l2object))
         return GST_STATE_CHANGE_FAILURE;
